@@ -100,6 +100,14 @@ static unsigned char tohex[16] = {
 };
 
 
+static gboolean
+is (const char *str, const char *value, size_t n)
+{
+	return g_ascii_strncasecmp (str, value, n) == 0 &&
+		(str[n] == '\0' || is_lwsp (str[n]));
+}
+
+
 /**
  * g_mime_content_encoding_from_string:
  * @str: a string representing a Content-Transfer-Encoding value
@@ -113,25 +121,28 @@ static unsigned char tohex[16] = {
 GMimeContentEncoding
 g_mime_content_encoding_from_string (const char *str)
 {
-	if (!g_ascii_strcasecmp (str, "7bit"))
+	while (is_lwsp (*str))
+		str++;
+	
+	if (is (str, "7bit", 4))
 		return GMIME_CONTENT_ENCODING_7BIT;
-	else if (!g_ascii_strcasecmp (str, "8bit"))
+	else if (is (str, "8bit", 4))
 		return GMIME_CONTENT_ENCODING_8BIT;
-	else if (!g_ascii_strcasecmp (str, "7-bit"))
+	else if (is (str, "7-bit", 5))
 		return GMIME_CONTENT_ENCODING_7BIT;
-	else if (!g_ascii_strcasecmp (str, "8-bit"))
+	else if (is (str, "8-bit", 5))
 		return GMIME_CONTENT_ENCODING_8BIT;
-	else if (!g_ascii_strcasecmp (str, "binary"))
+	else if (is (str, "binary", 6))
 		return GMIME_CONTENT_ENCODING_BINARY;
-	else if (!g_ascii_strcasecmp (str, "base64"))
+	else if (is (str, "base64", 6))
 		return GMIME_CONTENT_ENCODING_BASE64;
-	else if (!g_ascii_strcasecmp (str, "quoted-printable"))
+	else if (is (str, "quoted-printable", 16))
 		return GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE;
-	else if (!g_ascii_strcasecmp (str, "uuencode"))
+	else if (is (str, "uuencode", 8))
 		return GMIME_CONTENT_ENCODING_UUENCODE;
-	else if (!g_ascii_strcasecmp (str, "x-uuencode"))
+	else if (is (str, "x-uuencode", 10))
 		return GMIME_CONTENT_ENCODING_UUENCODE;
-	else if (!g_ascii_strcasecmp (str, "x-uue"))
+	else if (is (str, "x-uue", 5))
 		return GMIME_CONTENT_ENCODING_UUENCODE;
 	else
 		return GMIME_CONTENT_ENCODING_DEFAULT;
@@ -511,60 +522,47 @@ g_mime_encoding_base64_decode_step (const unsigned char *inbuf, size_t inlen, un
 	register unsigned char *outptr;
 	const unsigned char *inend;
 	register guint32 saved;
-	unsigned char c;
-	int npad, n, i;
+	unsigned char last[2];
+	unsigned char c, rank;
+	int n;
 	
 	inend = inbuf + inlen;
 	outptr = outbuf;
 	inptr = inbuf;
 	
-	npad = (*state >> 8) & 0xff;
-	n = *state & 0xff;
 	saved = *save;
+	n = *state;
+	
+	if (n < 0) {
+		last[0] = '=';
+		n = -n;
+	} else {
+		last[0] = '\0';
+	}
+	
+	last[1] = '\0';
 	
 	/* convert 4 base64 bytes to 3 normal bytes */
 	while (inptr < inend) {
-		c = gmime_base64_rank[*inptr++];
-		if (c != 0xff) {
-			saved = (saved << 6) | c;
+		rank = gmime_base64_rank[(c = *inptr++)];
+		if (rank != 0xff) {
+			saved = (saved << 6) | rank;
+			last[1] = last[0];
+			last[0] = c;
 			n++;
 			if (n == 4) {
 				*outptr++ = saved >> 16;
-				*outptr++ = saved >> 8;
-				*outptr++ = saved;
+				if (last[1] != '=')
+					*outptr++ = saved >> 8;
+				if (last[0] != '=')
+					*outptr++ = saved;
 				n = 0;
-				
-				if (npad > 0) {
-					outptr -= npad;
-					npad = 0;
-				}
 			}
 		}
 	}
 	
-	/* quickly scan back for '=' on the end somewhere */
-	/* fortunately we can drop 1 output char for each trailing '=' (up to 2) */
-	for (i = 2; inptr > inbuf && i; ) {
-		inptr--;
-		if (gmime_base64_rank[*inptr] != 0xff) {
-			if (*inptr == '=' && outptr > outbuf) {
-				if (n == 0) {
-					/* we've got a complete quartet so it's
-					   safe to drop an output character. */
-					outptr--;
-				} else if (npad < 2) {
-					/* keep a record of the number of ='s at
-					   the end of the input stream, up to 2 */
-					npad++;
-				}
-			}
-			
-			i--;
-		}
-	}
-	
-	*state = (npad << 8) | n;
-	*save = n ? saved : 0;
+	*state = last[0] == '=' ? -n : n;
+	*save = saved;
 	
 	return (outptr - outbuf);
 }
@@ -687,7 +685,7 @@ g_mime_encoding_uuencode_step (const unsigned char *inbuf, size_t inlen, unsigne
 	i = *state & 0xff;
 	uulen = (*state >> 8) & 0xff;
 	
-	if ((inlen + uulen) < 45) {
+	if ((inlen + i + uulen) < 45) {
 		/* not enough input to write a full uuencoded line */
 		bufptr = uubuf + ((uulen / 3) * 4);
 	} else {
@@ -898,8 +896,7 @@ g_mime_encoding_quoted_encode_close (const unsigned char *inbuf, size_t inlen, u
 	
 	last = *state;
 	if (last != -1) {
-		/* space/tab must be encoded if its the last character on
-		   the line */
+		/* space/tab must be encoded if its the last character on the line */
 		if (is_qpsafe (last) && !is_blank (last)) {
 			*outptr++ = last;
 		} else {
@@ -907,17 +904,15 @@ g_mime_encoding_quoted_encode_close (const unsigned char *inbuf, size_t inlen, u
 			*outptr++ = tohex[(last >> 4) & 0xf];
 			*outptr++ = tohex[last & 0xf];
 		}
-	}
-	
-	if (last != '\n') {
+		
 		/* we end with =\n so that the \n isn't interpreted as a real
 		   \n when it gets decoded later */
 		*outptr++ = '=';
 		*outptr++ = '\n';
+		*state = -1;
 	}
 	
 	*save = 0;
-	*state = -1;
 	
 	return (outptr - outbuf);
 }
