@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2013 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -138,7 +138,7 @@ filter_filter (GMimeFilter *filter, char *inbuf, size_t inlen, size_t prespace,
 			
 			while (inptr < inend) {
 				left = inend - inptr;
-				if (left < 6) {
+				if (left <= 6) {
 					if (!strncmp (inptr, "begin ", left))
 						g_mime_filter_backup (filter, inptr, left);
 					break;
@@ -174,7 +174,10 @@ filter_filter (GMimeFilter *filter, char *inbuf, size_t inlen, size_t prespace,
 			break;
 		default:
 			/* either we haven't seen the begin-line or we've finished decoding */
-			goto done;
+			*outprespace = filter->outpre;
+			*outlen = nwritten;
+			*outbuf = inbuf;
+			return;
 		}
 	}
 	
@@ -183,7 +186,6 @@ filter_filter (GMimeFilter *filter, char *inbuf, size_t inlen, size_t prespace,
 	nwritten = g_mime_encoding_step (encoder, inbuf, inlen, filter->outbuf);
 	g_assert (nwritten <= len);
 	
- done:
 	*outprespace = filter->outpre;
 	*outbuf = filter->outbuf;
 	*outlen = nwritten;
@@ -245,7 +247,7 @@ g_mime_filter_basic_new (GMimeContentEncoding encoding, gboolean encode)
 {
 	GMimeFilterBasic *basic;
 	
-	basic = g_object_newv (GMIME_TYPE_FILTER_BASIC, 0, NULL);
+	basic = g_object_new (GMIME_TYPE_FILTER_BASIC, NULL);
 	
 	if (encode)
 		g_mime_encoding_init_encode (&basic->encoder, encoding);
