@@ -72,19 +72,18 @@ static void to_list_changed (InternetAddressList *list, gpointer args, GMimeMess
 static void cc_list_changed (InternetAddressList *list, gpointer args, GMimeMessage *message);
 static void bcc_list_changed (InternetAddressList *list, gpointer args, GMimeMessage *message);
 
-
 static GMimeObjectClass *parent_class = NULL;
 
 static struct {
 	const char *name;
 	GMimeEventCallback changed_cb;
 } address_types[] = {
-	{ "Sender",   (GMimeEventCallback) sender_changed   },
-	{ "From",     (GMimeEventCallback) from_changed     },
-	{ "Reply-To", (GMimeEventCallback) reply_to_changed },
-	{ "To",       (GMimeEventCallback) to_list_changed  },
-	{ "Cc",       (GMimeEventCallback) cc_list_changed  },
-	{ "Bcc",      (GMimeEventCallback) bcc_list_changed }
+	{ "Sender",          (GMimeEventCallback) sender_changed          },
+	{ "From",            (GMimeEventCallback) from_changed            },
+	{ "Reply-To",        (GMimeEventCallback) reply_to_changed        },
+	{ "To",              (GMimeEventCallback) to_list_changed         },
+	{ "Cc",              (GMimeEventCallback) cc_list_changed         },
+	{ "Bcc",             (GMimeEventCallback) bcc_list_changed        },
 };
 
 #define N_ADDRESS_TYPES G_N_ELEMENTS (address_types)
@@ -257,7 +256,7 @@ static const char *message_headers[] = {
 	"Subject",
 	"Date",
 	"Message-Id",
-	"MIME-Version",
+	"MIME-Version"
 };
 
 static void
