@@ -45,7 +45,6 @@
  **/
 
 #define BLOCK_BUFFER_LEN   4096
-#define BUFFER_GROW_SIZE   1024  /* should this also be 4k? */
 
 static void g_mime_stream_buffer_class_init (GMimeStreamBufferClass *klass);
 static void g_mime_stream_buffer_init (GMimeStreamBuffer *stream, GMimeStreamBufferClass *klass);
@@ -127,12 +126,9 @@ g_mime_stream_buffer_init (GMimeStreamBuffer *stream, GMimeStreamBufferClass *kl
 static void
 g_mime_stream_buffer_finalize (GObject *object)
 {
-	GMimeStreamBuffer *stream = (GMimeStreamBuffer *) object;
+	GMimeStream *stream = (GMimeStream *) object;
 	
-	if (stream->source)
-		g_object_unref (stream->source);
-	
-	g_free (stream->buffer);
+	stream_close (stream);
 	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -296,7 +292,6 @@ stream_close (GMimeStream *stream)
 	if (buffer->source == NULL)
 		return 0;
 	
-	g_mime_stream_close (buffer->source);
 	g_object_unref (buffer->source);
 	buffer->source = NULL;
 	
