@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -139,6 +139,7 @@ static void
 g_mime_stream_filter_init (GMimeStreamFilter *stream, GMimeStreamFilterClass *klass)
 {
 	stream->source = NULL;
+	stream->owner = FALSE;
 	stream->priv = g_new (struct _GMimeStreamFilterPrivate, 1);
 	stream->priv->filters = NULL;
 	stream->priv->filterid = 0;
@@ -287,7 +288,7 @@ stream_flush (GMimeStream *stream)
 	if (len > 0 && g_mime_stream_write (filter->source, buffer, len) == -1)
 		return -1;
 	
-	return g_mime_stream_flush (filter->source);
+	return 0;
 }
 
 static int
@@ -299,7 +300,10 @@ stream_close (GMimeStream *stream)
 	if (!priv->last_was_read)
 		stream_flush (stream);
 	
-	return g_mime_stream_close (filter->source);
+	if (filter->owner)
+		return g_mime_stream_close (filter->source);
+	
+	return 0;
 }
 
 static gboolean
@@ -364,7 +368,7 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 	GMimeStreamFilter *filter = (GMimeStreamFilter *) stream;
 	GMimeStreamFilter *sub;
 	
-	sub = g_object_newv (GMIME_TYPE_STREAM_FILTER, 0, NULL);
+	sub = g_object_new (GMIME_TYPE_STREAM_FILTER, NULL);
 	sub->source = filter->source;
 	g_object_ref (sub->source);
 	
@@ -392,9 +396,9 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 		sub->priv->filterid = filter->priv->filterid;
 	}
 	
-	g_mime_stream_construct (GMIME_STREAM (filter), start, end);
+	g_mime_stream_construct ((GMimeStream *) filter, start, end);
 	
-	return GMIME_STREAM (sub);
+	return (GMimeStream *) sub;
 }
 
 
@@ -414,15 +418,15 @@ g_mime_stream_filter_new (GMimeStream *stream)
 	
 	g_return_val_if_fail (GMIME_IS_STREAM (stream), NULL);
 	
-	filter = g_object_newv (GMIME_TYPE_STREAM_FILTER, 0, NULL);
+	filter = g_object_new (GMIME_TYPE_STREAM_FILTER, NULL);
 	filter->source = stream;
 	g_object_ref (stream);
 	
-	g_mime_stream_construct (GMIME_STREAM (filter),
+	g_mime_stream_construct ((GMimeStream *) filter,
 				 stream->bound_start,
 				 stream->bound_end);
 	
-	return GMIME_STREAM (filter);
+	return (GMimeStream *) filter;
 }
 
 
@@ -497,3 +501,40 @@ g_mime_stream_filter_remove (GMimeStreamFilter *stream, int id)
 		f = f->next;
 	}
 }
+
+
+/**
+ * g_mime_stream_filter_set_owner:
+ * @stream: a #GMimeStreamFilter
+ * @owner: %TRUE if this stream should own the source stream or %FALSE otherwise
+ *
+ * Sets whether or not @stream owns the source stream..
+ *
+ * Note: @owner should be %TRUE if the stream should close() the
+ * backend source stream when destroyed or %FALSE otherwise.
+ **/
+void
+g_mime_stream_filter_set_owner (GMimeStreamFilter *stream, gboolean owner)
+{
+	g_return_if_fail (GMIME_IS_STREAM_FILTER (stream));
+	
+	stream->owner = owner;
+}
+
+
+/**
+ * g_mime_stream_filter_get_owner:
+ * @stream: a #GMimeStreamFilter
+ *
+ * Gets whether or not @stream owns the source stream.
+ *
+ * Returns: %TRUE if @stream owns the source stream or %FALSE
+ * otherwise.
+ **/
+gboolean
+g_mime_stream_filter_get_owner (GMimeStreamFilter *stream)
+{
+	g_return_val_if_fail (GMIME_IS_STREAM_FILTER (stream), FALSE);
+	
+	return stream->owner;
+}
