@@ -892,6 +892,10 @@ test_ah_message_parse (void)
 						g_error_free (err);
 						throw (ex);
 					}
+#else
+					/* pretend that we do not expect any gossip so that we don't fail the test
+					   simply because we don't have crypto support. */
+					gossip_expected = FALSE;
 #endif
 				}
 			}
@@ -1066,9 +1070,11 @@ int main (int argc, char **argv)
 	test_ah_message_parse ();
 	testsuite_end ();
 	
+#ifdef ENABLE_CRYPTO
 	testsuite_start ("Autocrypt: inject headers");
 	test_ah_injection ();
 	testsuite_end ();
+#endif
 	
 	g_mime_shutdown ();
 	
