@@ -85,7 +85,7 @@ streams_match (GMimeStream **streams, const char *filename)
 		totalsize = len - (streams[0]->position - streams[0]->bound_start);
 	}
 	
-	while (totalread < totalsize) {
+	while (totalread <= totalsize) {
 		if ((n = g_mime_stream_read (streams[0], buf, sizeof (buf))) <= 0)
 			break;
 		
@@ -259,13 +259,41 @@ check_stream_fs (const char *input, const char *output, const char *filename, gi
 	}
 	
 	stream = g_mime_stream_fs_new (fd[0]);
+	if (!g_mime_stream_fs_get_owner ((GMimeStreamFs *) stream)) {
+		ex = exception_new ("GMimeStreamFs is not the owner `%s'", filename);
+		g_object_unref (stream);
+		close (fd[1]);
+		throw (ex);
+	}
+	
 	streams[0] = g_mime_stream_substream (stream, start, end);
 	g_object_unref (stream);
+	if (g_mime_stream_fs_get_owner ((GMimeStreamFs *) streams[0])) {
+		ex = exception_new ("GMimeStreamFs substream should not be the owner `%s'", filename);
+		g_object_unref (stream);
+		close (fd[1]);
+		throw (ex);
+	}
 	
-	streams[1] = g_mime_stream_fs_new (fd[1]);
+	streams[1] = g_mime_stream_pipe_new (fd[1]);
 	
-	if (!streams_match (streams, filename))
+	if (!streams_match (streams, filename)) {
 		ex = exception_new ("GMimeStreamFs streams did not match for `%s'", filename);
+		goto cleanup;
+	}
+	
+	if (!g_mime_stream_eos (streams[0])) {
+		ex = exception_new ("GMimeStreamFs is not at the end-of-stream `%s'", filename);
+		goto cleanup;
+	}
+	
+	g_mime_stream_reset (streams[0]);
+	if (g_mime_stream_eos (streams[0])) {
+		ex = exception_new ("GMimeStreamFs did not properly reset `%s'", filename);
+		goto cleanup;
+	}
+	
+cleanup:
 	
 	g_object_unref (streams[0]);
 	g_object_unref (streams[1]);
@@ -292,13 +320,41 @@ check_stream_file (const char *input, const char *output, const char *filename,
 	}
 	
 	stream = g_mime_stream_file_new (fp[0]);
+	if (!g_mime_stream_file_get_owner ((GMimeStreamFile *) stream)) {
+		ex = exception_new ("GMimeStreamFile is not the owner `%s'", filename);
+		g_object_unref (stream);
+		fclose (fp[1]);
+		throw (ex);
+	}
+	
 	streams[0] = g_mime_stream_substream (stream, start, end);
 	g_object_unref (stream);
+	if (g_mime_stream_file_get_owner ((GMimeStreamFile *) streams[0])) {
+		ex = exception_new ("GMimeStreamFile substream should not be the owner `%s'", filename);
+		g_object_unref (stream);
+		fclose (fp[1]);
+		throw (ex);
+	}
 	
 	streams[1] = g_mime_stream_file_new (fp[1]);
 	
-	if (!streams_match (streams, filename))
+	if (!streams_match (streams, filename)) {
 		ex = exception_new ("GMimeStreamFile streams did not match for `%s'", filename);
+		goto cleanup;
+	}
+	
+	if (!g_mime_stream_eos (streams[0])) {
+		ex = exception_new ("GMimeStreamFile is not at the end-of-stream `%s'", filename);
+		goto cleanup;
+	}
+	
+	g_mime_stream_reset (streams[0]);
+	if (g_mime_stream_eos (streams[0])) {
+		ex = exception_new ("GMimeStreamFile did not properly reset `%s'", filename);
+		goto cleanup;
+	}
+	
+cleanup:
 	
 	g_object_unref (streams[0]);
 	g_object_unref (streams[1]);
@@ -326,13 +382,41 @@ check_stream_mmap (const char *input, const char *output, const char *filename,
 	}
 	
 	stream = g_mime_stream_mmap_new (fd[0], PROT_READ, MAP_PRIVATE);
+	if (!g_mime_stream_mmap_get_owner ((GMimeStreamMmap *) stream)) {
+		ex = exception_new ("GMimeStreamMmap is not the owner `%s'", filename);
+		g_object_unref (stream);
+		close (fd[1]);
+		throw (ex);
+	}
+	
 	streams[0] = g_mime_stream_substream (stream, start, end);
 	g_object_unref (stream);
+	if (g_mime_stream_mmap_get_owner ((GMimeStreamMmap *) streams[0])) {
+		ex = exception_new ("GMimeStreamMmap substream should not be the owner `%s'", filename);
+		g_object_unref (stream);
+		close (fd[1]);
+		throw (ex);
+	}
 	
 	streams[1] = g_mime_stream_mmap_new (fd[1], PROT_READ, MAP_PRIVATE);
 	
-	if (!streams_match (streams, filename))
-		ex = exception_new ("streams did not match");
+	if (!streams_match (streams, filename)) {
+		ex = exception_new ("GMimeStreamMmap streams did not match for `%s'", filename);
+		goto cleanup;
+	}
+	
+	if (!g_mime_stream_eos (streams[0])) {
+		ex = exception_new ("GMimeStreamMmap is not at the end-of-stream `%s'", filename);
+		goto cleanup;
+	}
+	
+	g_mime_stream_reset (streams[0]);
+	if (g_mime_stream_eos (streams[0])) {
+		ex = exception_new ("GMimeStreamMmap did not properly reset `%s'", filename);
+		goto cleanup;
+	}
+	
+cleanup:
 	
 	g_object_unref (streams[0]);
 	g_object_unref (streams[1]);
@@ -384,6 +468,7 @@ check_stream_gio (const char *input, const char *output, const char *filename, g
 {
 	GMimeStream *streams[2];
 	Exception *ex = NULL;
+	GMimeStream *stream;
 	GFile *file;
 	int fd;
 	
@@ -395,11 +480,42 @@ check_stream_gio (const char *input, const char *output, const char *filename, g
 		return FALSE;
 	}
 	
-	streams[0] = g_mime_stream_gio_new_with_bounds (file, start, end);
+	stream = g_mime_stream_gio_new (file);
+	if (!g_mime_stream_gio_get_owner ((GMimeStreamGIO *) stream)) {
+		ex = exception_new ("GMimeStreamGIO is not the owner `%s'", filename);
+		g_object_unref (stream);
+		close (fd);
+		throw (ex);
+	}
+	
+	streams[0] = g_mime_stream_substream (stream, start, end);
+	g_object_unref (stream);
+	if (g_mime_stream_gio_get_owner ((GMimeStreamGIO *) streams[0])) {
+		ex = exception_new ("GMimeStreamGIO substream should not be the owner `%s'", filename);
+		g_object_unref (stream);
+		close (fd);
+		throw (ex);
+	}
+	
 	streams[1] = g_mime_stream_fs_new (fd);
 	
-	if (!streams_match (streams, filename))
+	if (!streams_match (streams, filename)) {
 		ex = exception_new ("GMimeStreamGIO streams did not match for `%s'", filename);
+		goto cleanup;
+	}
+	
+	if (!g_mime_stream_eos (streams[0])) {
+		ex = exception_new ("GMimeStreamGIO is not at the end-of-stream `%s'", filename);
+		goto cleanup;
+	}
+	
+	g_mime_stream_reset (streams[0]);
+	if (g_mime_stream_eos (streams[0])) {
+		ex = exception_new ("GMimeStreamGIO did not properly reset `%s'", filename);
+		goto cleanup;
+	}
+	
+cleanup:
 	
 	g_object_unref (streams[0]);
 	g_object_unref (streams[1]);
